import re
import traceback
from typing import AbstractSet, List, Mapping, Optional, Sequence, Tuple

from typing_extensions import TypedDict

from zulint.printer import BLUE, ENDC, GREEN, MAGENTA, YELLOW, colors, print_err

Rule = TypedDict("Rule", {
    "bad_lines": Sequence[str],
    "description": str,
    "exclude": AbstractSet[str],
    "exclude_line": AbstractSet[Tuple[str, str]],
    "exclude_pattern": str,
    "good_lines": Sequence[str],
    "include_only": AbstractSet[str],
    "pattern": str,
    "strip": str,
    "strip_rule": str,
}, total=False)
LineTup = Tuple[int, str, str, str]


class RuleList:
    """Defines and runs custom linting rules for the specified language."""

    def __init__(
        self,
        langs: Sequence[str],
        rules: Sequence[Rule],
        max_length: Optional[int] = None,
        length_exclude: AbstractSet[str] = set(),
        shebang_rules: Sequence[Rule] = [],
        exclude_files_in: Optional[str] = None,
        exclude_max_length_fns: Sequence[str] = [],
        exclude_max_length_line_patterns: Sequence[str] = [],
    ) -> None:
        self.langs = langs
        self.rules = rules
        self.max_length = max_length
        self.length_exclude = length_exclude
        self.shebang_rules = shebang_rules
        # Exclude the files in this folder from rules
        self.exclude_files_in = "\\"
        self.verbose = False

        # Exclude some file names and line patterns from max line length
        # These defaults are from https://github.com/zulip/zulip repository.
        self.exclude_max_length_fns = exclude_max_length_fns
        self.exclude_max_length_line_patterns = exclude_max_length_line_patterns

    def get_line_info_from_file(self, fn: str) -> List[LineTup]:
        line_tups = []
        with open(fn, encoding='utf8') as f:
            for i, line in enumerate(f):
                line_newline_stripped = line.strip('\n')
                line_fully_stripped = line_newline_stripped.strip()
                if line_fully_stripped.endswith('  # nolint'):
                    continue
                tup = (i, line, line_newline_stripped, line_fully_stripped)
                line_tups.append(tup)
        return line_tups

    def get_rules_applying_to_fn(self, fn: str, rules: Sequence[Rule]) -> List[Rule]:
        rules_to_apply = []
        for rule in rules:
            excluded = False
            for item in rule.get('exclude', set()):
                if fn.startswith(item):
                    excluded = True
                    break
            if excluded:
                continue
            if rule.get("include_only"):
                found = False
                for item in rule.get("include_only", set()):
                    if item in fn:
                        found = True
                if not found:
                    continue
            rules_to_apply.append(rule)

        return rules_to_apply

    def check_file_for_pattern(
        self,
        fn: str,
        line_tups: Sequence[LineTup],
        identifier: str,
        color: str,
        rule: Rule,
    ) -> bool:

        '''
        DO NOT MODIFY THIS FUNCTION WITHOUT PROFILING.

        This function gets called ~40k times, once per file per regex.

        Inside it's doing a regex check for every line in the file, so
        it's important to do things like pre-compiling regexes.

        DO NOT INLINE THIS FUNCTION.

        We need to see it show up in profiles, and the function call
        overhead will never be a bottleneck.
        '''
        exclude_lines = {
            line for
            (exclude_fn, line) in rule.get('exclude_line', set())
            if exclude_fn == fn
        }
        unmatched_exclude_lines = exclude_lines.copy()

        pattern = re.compile(rule['pattern'])
        strip_rule = rule.get('strip')  # type: Optional[str]

        ok = True
        for (i, line, line_newline_stripped, line_fully_stripped) in line_tups:
            if line_fully_stripped in exclude_lines:
                unmatched_exclude_lines.discard(line_fully_stripped)
                continue
            try:
                line_to_check = line_fully_stripped
                if strip_rule is not None:
                    if strip_rule == '\n':
                        line_to_check = line_newline_stripped
                    else:
                        raise Exception("Invalid strip rule")
                if pattern.search(line_to_check):
                    if rule.get("exclude_pattern"):
                        if re.search(rule['exclude_pattern'], line_to_check):
                            continue
                    self.print_error(rule, line, identifier, color, fn, i+1)
                    ok = False
            except Exception:
                print("Exception with %s at %s line %s" % (rule['pattern'], fn, i+1))
                traceback.print_exc()

        if unmatched_exclude_lines:
            print('Please remove exclusions for file %s: %s' % (fn, unmatched_exclude_lines))

        return ok

    def print_error(
        self,
        rule: Rule,
        line: str,
        identifier: str,
        color: str,
        fn: str,
        line_number: int,
    ) -> None:
        print_err(identifier, color, '{} {}at {} line {}:'.format(
            YELLOW + rule['description'], BLUE, fn, line_number))
        print_err(identifier, color, line)
        if self.verbose:
            if rule.get('good_lines'):
                print_err(identifier, color, GREEN + "  Good code: {}{}".format(
                    (YELLOW + " | " + GREEN).join(rule['good_lines']), ENDC))
            if rule.get('bad_lines'):
                print_err(identifier, color, MAGENTA + "  Bad code:  {}{}".format(
                    (YELLOW + " | " + MAGENTA).join(rule['bad_lines']), ENDC))
            print_err(identifier, color, "")

    def check_file_for_long_lines(
        self,
        fn: str,
        max_length: int,
        line_tups: Sequence[LineTup]
    ) -> bool:
        ok = True
        for (i, line, line_newline_stripped, line_fully_stripped) in line_tups:
            line_length = len(line)
            if (line_length > max_length and
                not re.findall(r'|'.join(self.exclude_max_length_fns), fn) and
                not re.findall(r'|'.join(self.exclude_max_length_line_patterns), line)):
                print("Line too long (%s) at %s line %s: %s" % (len(line), fn, i+1, line_newline_stripped))
                ok = False
        return ok

    def custom_check_file(
        self,
        fn: str,
        identifier: str,
        color: str,
        max_length: Optional[int] = None,
    ) -> bool:
        failed = False

        line_tups = self.get_line_info_from_file(fn=fn)

        rules_to_apply = self.get_rules_applying_to_fn(fn=fn, rules=self.rules)

        for rule in rules_to_apply:
            ok = self.check_file_for_pattern(
                fn=fn,
                line_tups=line_tups,
                identifier=identifier,
                color=color,
                rule=rule,
            )
            if not ok:
                failed = True

        # TODO: Move the below into more of a framework.
        firstline = None
        lastLine = None
        if line_tups:
            # line_fully_stripped for the first line.
            firstline = line_tups[0][3]
            lastLine = line_tups[-1][1]

        if max_length is not None:
            ok = self.check_file_for_long_lines(
                fn=fn,
                max_length=max_length,
                line_tups=line_tups,
            )
            if not ok:
                failed = True

        if firstline:
            shebang_rules_to_apply = self.get_rules_applying_to_fn(fn=fn, rules=self.shebang_rules)
            for rule in shebang_rules_to_apply:
                if re.search(rule['pattern'], firstline):
                    self.print_error(rule, firstline, identifier, color, fn, 1)
                    failed = True

        if lastLine and ('\n' not in lastLine):
            print("No newline at the end of file. Fix with `sed -i '$a\\' %s`" % (fn,))
            failed = True

        return failed

    def check(self, by_lang: Mapping[str, Sequence[str]], verbose: bool = False) -> bool:
        # By default, a rule applies to all files within the extension for
        # which it is specified (e.g. all .py files)
        # There are three operators we can use to manually include or exclude files from linting for a rule:
        # 'exclude': 'set([<path>, ...])' - if <path> is a filename, excludes that file.
        #                                   if <path> is a directory, excludes all files
        #                                   directly below the directory <path>.
        # 'exclude_line': 'set([(<path>, <line>), ...])' - excludes all lines matching <line>
        #                                                  in the file <path> from linting.
        # 'include_only': 'set([<path>, ...])' - includes only those files where <path> is a
        #                                        substring of the filepath.
        failed = False
        self.verbose = verbose
        for lang in self.langs:
            color = next(colors)
            for fn in by_lang[lang]:
                if fn.startswith(self.exclude_files_in) or ('custom_check.py' in fn):
                    # This is a bit of a hack, but it generally really doesn't
                    # work to check the file that defines all the things to check for.
                    #
                    # TODO: Migrate this to looking at __module__ type attributes.
                    continue
                max_length = None
                if fn not in self.length_exclude:
                    max_length = self.max_length
                if self.custom_check_file(fn, lang, color, max_length=max_length):
                    failed = True

        return failed



import argparse
import signal
import subprocess
from typing import Callable, Sequence, Tuple

from zulint.printer import colors, print_err


def run_command(
    name: str,
    color: str,
    command: Sequence[str],
    suppress_line: Callable[[str], bool] = lambda line: False,
) -> int:
    with subprocess.Popen(
        command,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        universal_newlines=True,
    ) as p:
        assert p.stdout is not None
        for line in iter(p.stdout.readline, ""):
            if not suppress_line(line):
                print_err(name, color, line)
        if p.wait() < 0:
            try:
                signal_name = signal.Signals(-p.returncode).name
            except (AttributeError, ValueError):
                signal_name = "signal {}".format(-p.returncode)
            print_err(name, color, "{} terminated by {}".format(command[0], signal_name))
        return p.returncode


def run_pycodestyle(files: Sequence[str], ignored_rules: Sequence[str]) -> bool:
    if len(files) == 0:
        return False

    command = [
        "pycodestyle",
        "--ignore={rules}".format(rules=",".join(ignored_rules)),
        "--",
        *files,
    ]
    return run_command("pep8", next(colors), command) != 0


def run_pyflakes(
    files: Sequence[str],
    options: argparse.Namespace,
    suppress_patterns: Sequence[Tuple[str, str]] = [],
) -> bool:
    if len(files) == 0:
        return False
    failed = False
    color = next(colors)
    with subprocess.Popen(
        ['pyflakes', '--', *files],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        universal_newlines=True,
    ) as pyflakes:
        # Implied by use of subprocess.PIPE
        assert pyflakes.stdout is not None
        assert pyflakes.stderr is not None

        def suppress_line(line: str) -> bool:
            for file_pattern, line_pattern in suppress_patterns:
                if file_pattern in line and line_pattern in line:
                    return True
            return False

        for ln in pyflakes.stdout.readlines() + pyflakes.stderr.readlines():
            if not suppress_line(ln):
                print_err('pyflakes', color, ln)
                failed = True
    return failed



import argparse
import logging
import multiprocessing
import sys
import weakref
from typing import Callable, Dict, List, Mapping, NoReturn, Sequence, Set, Tuple, Union

from zulint import lister
from zulint.linters import run_command
from zulint.printer import BLUE, BOLDRED, ENDC, GREEN, colors


def add_default_linter_arguments(parser: argparse.ArgumentParser) -> None:
    parser.add_argument('--modified', '-m',
                        action='store_true',
                        help='Only check modified files')
    parser.add_argument('--verbose-timing', '-vt',
                        action='store_true',
                        help='Print verbose timing output')
    parser.add_argument('targets',
                        nargs='*',
                        help='Specify directories to check')
    parser.add_argument('--skip',
                        default=[],
                        type=split_arg_into_list,
                        help='Specify linters to skip, eg: --skip=mypy,gitlint')
    parser.add_argument('--only',
                        default=[],
                        type=split_arg_into_list,
                        help='Specify linters to run, eg: --only=mypy,gitlint')
    parser.add_argument('--list', '-l',
                        action='store_true',
                        help='List all the registered linters')
    parser.add_argument('--list-groups', '-lg',
                        action='store_true',
                        help='List all the registered linter groups')
    parser.add_argument('--groups', '-g',
                        default=[],
                        type=split_arg_into_list,
                        help='Only run linter for languages in the group(s), e.g.: '
                        '--groups=backend,frontend')
    parser.add_argument('--verbose', '-v',
                        action='store_true',
                        help='Print verbose output where available')
    parser.add_argument('--fix',
                        action='store_true',
                        help='Automatically fix problems where supported')
    parser.add_argument('--jobs', '-j',
                        type=int,
                        help='Limit number of parallel jobs')

def split_arg_into_list(arg: str) -> List[str]:
    return [linter for linter in arg.split(',')]

run_parallel_functions = weakref.WeakValueDictionary()  # type: weakref.WeakValueDictionary[int, Callable[[], int]]

def run_parallel_worker(item: Tuple[str, int]) -> Tuple[str, int]:
    name, func_id = item
    func = run_parallel_functions[func_id]
    logging.info("start " + name)
    result = func()
    logging.info("finish " + name)
    sys.stdout.flush()
    sys.stderr.flush()
    return name, result

def run_parallel(lint_functions: Mapping[str, Callable[[], int]], jobs: int) -> Set[str]:
    # Smuggle the functions through a global variable to work around
    # multiprocessing's inability to pickle closures.
    for func in lint_functions.values():
        run_parallel_functions[id(func)] = func

    failed_linters = set()
    args = ((name, id(func)) for name, func in lint_functions.items())
    if jobs != 1 and multiprocessing.get_start_method() == "fork":
        with multiprocessing.Pool(jobs) as pool:
            for name, result in pool.imap_unordered(run_parallel_worker, args):
                if result != 0:
                    failed_linters.add(name)
    else:
        for name, result in map(run_parallel_worker, args):
            if result != 0:
                failed_linters.add(name)
    return failed_linters

class LinterConfig:
    lint_functions = {}  # type: Dict[str, Callable[[], int]]
    lint_descriptions = {}  # type: Dict[str, str]
    fixable_linters = set()  # type: Set[str]

    def __init__(self, args: argparse.Namespace) -> None:
        self.args = args
        self.by_lang = {}  # type: Dict[str, List[str]]
        self.groups = {}  # type: Mapping[str, Sequence[str]]

    def list_files(
        self,
        file_types: Sequence[str] = [],
        groups: Mapping[str, Sequence[str]] = {},
        use_shebang: bool = True,
        exclude: Sequence[str] = [],
    ) -> Dict[str, List[str]]:
        assert file_types or groups, "Atleast one of `file_types` or `groups` must be specified."

        self.groups = groups
        if self.args.groups:
            file_types = [ft for group in self.args.groups for ft in groups[group]]
        else:
            file_types = [*file_types, *{ft for group in groups.values() for ft in group}]

        self.by_lang = lister.list_files(
            targets=self.args.targets, modified_only=self.args.modified,
            ftypes=file_types, use_shebang=use_shebang,
            group_by_ftype=True, exclude=exclude,
        )
        return self.by_lang

    def lint(self, func: Callable[[], int]) -> Callable[[], int]:
        self.lint_functions[func.__name__] = func
        self.lint_descriptions[func.__name__] = func.__doc__ if func.__doc__ else "External Linter"
        return func

    def external_linter(
        self,
        name: str,
        command: Sequence[str],
        target_langs: Sequence[str] = [],
        pass_targets: bool = True,
        fix_arg: Union[str, Sequence[str]] = [],
        description: str = "External Linter",
        check_arg: Union[str, Sequence[str]] = [],
        suppress_line: Callable[[str], bool] = lambda line: False,
    ) -> None:
        """Registers an external linter program to be run as part of the
        linter.  This program will be passed the subset of files being
        linted that have extensions in target_langs.  If there are no
        such files, exits without doing anything.

        If target_langs is empty, just runs the linter unconditionally.
        """
        self.lint_descriptions[name] = description
        if fix_arg or check_arg:
            self.fixable_linters.add(name)
        color = next(colors)

        def run_linter() -> int:
            targets = []  # type: List[str]
            if len(target_langs) != 0:
                targets = [target for lang in target_langs for target in self.by_lang[lang]]
                if len(targets) == 0:
                    # If this linter has a list of languages, and
                    # no files in those languages are to be checked,
                    # then we can safely return success without
                    # invoking the external linter.
                    return 0

            full_command = list(command)
            arg = fix_arg if self.args.fix else check_arg
            full_command += [arg] if isinstance(arg, str) else arg

            if pass_targets:
                full_command += targets

            return run_command(name, color, full_command, suppress_line)

        self.lint_functions[name] = run_linter

    def set_logger(self) -> None:
        logging.basicConfig(format="%(asctime)s %(message)s")
        logger = logging.getLogger()
        if self.args.verbose_timing:
            logger.setLevel(logging.INFO)
        else:
            logger.setLevel(logging.WARNING)

    def do_lint(self) -> NoReturn:
        assert not self.args.only or not self.args.skip, "Only one of --only or --skip can be used at once."
        if self.args.only:
            self.lint_functions = {linter: self.lint_functions[linter] for linter in self.args.only}
        for linter in self.args.skip:
            del self.lint_functions[linter]
        if self.args.list:
            print("{}{:<15} {} {}".format(BOLDRED, 'Linter', 'Description', ENDC))
            for linter, desc in self.lint_descriptions.items():
                print("{}{:<15} {}{}{}".format(BLUE, linter, GREEN, desc, ENDC))
            sys.exit()
        if self.args.list_groups:
            print("{}{:<15} {} {}".format(BOLDRED, 'Linter Group', 'File types', ENDC))
            for group, file_types in self.groups.items():
                print("{}{:<15} {}{}{}".format(BLUE, group, GREEN, ", ".join(file_types), ENDC))
            sys.exit()
        self.set_logger()

        jobs = self.args.jobs
        if self.args.fix:
            # Do not run multiple fixers in parallel, since they might
            # race with each other and corrupt each other’s output.
            jobs = 1

        failed_linters = run_parallel(self.lint_functions, jobs)

        failed_fixable_linters = failed_linters & self.fixable_linters
        if failed_fixable_linters:
            print(
                "Run {}{} --fix{} to fix errors for the following linters: {}".format(
                    BLUE, sys.argv[0], ENDC, ",".join(sorted(failed_fixable_linters))
                ),
            )

        sys.exit(1 if failed_linters else 0)



from itertools import cycle

# Terminal Color codes for use in differentiatng linters
BOLDRED = '\x1B[1;31m'
GREEN = '\x1b[32m'
YELLOW = '\x1b[33m'
BLUE = '\x1b[34m'
MAGENTA = '\x1b[35m'
CYAN = '\x1b[36m'
ENDC = '\033[0m'

colors = cycle([GREEN, YELLOW, BLUE, MAGENTA, CYAN])

def print_err(name: str, color: str, line: str) -> None:
    print('{color}{name}{pad}|{end} {red_color}{line!s}{end}'.format(
        color=color,
        name=name,
        pad=' ' * max(0, 10 - len(name)),
        red_color=BOLDRED,
        line=line.rstrip(),
        end=ENDC,
    ), flush=True)



#!/usr/bin/env python3

import argparse
import os
import re
import subprocess
import sys
from collections import defaultdict
from typing import Dict, List, Sequence, Union, overload

from typing_extensions import Literal


def get_ftype(fpath: str, use_shebang: bool) -> str:
    ext = os.path.splitext(fpath)[1]
    if ext:
        return ext[1:]
    elif use_shebang:
        # opening a file may throw an OSError
        with open(fpath, encoding='utf8') as f:
            first_line = f.readline()
            if re.search(r'^#!.*\bpython', first_line):
                return 'py'
            elif re.search(r'^#!.*sh', first_line):
                return 'sh'
            elif re.search(r'^#!.*\bperl', first_line):
                return 'pl'
            elif re.search(r'^#!.*\bnode', first_line):
                return 'js'
            elif re.search(r'^#!.*\bruby', first_line):
                return 'rb'
            elif re.search(r'^#!.*\btail', first_line):
                return ''  # do not lint these scripts.
            elif re.search(r'^#!', first_line):
                print('Error: Unknown shebang in file "%s":\n%s' % (fpath, first_line), file=sys.stderr)
                return ''
            else:
                return ''
    else:
        return ''

@overload
def list_files(
    group_by_ftype: Literal[False] = False,
    targets: Sequence[str] = [],
    ftypes: Sequence[str] = [],
    use_shebang: bool = True,
    modified_only: bool = False,
    exclude: Sequence[str] = [],
    extless_only: bool = False,
) -> List[str]:
    ...

@overload
def list_files(
    group_by_ftype: Literal[True],
    targets: Sequence[str] = [],
    ftypes: Sequence[str] = [],
    use_shebang: bool = True,
    modified_only: bool = False, exclude: Sequence[str] = [],
    extless_only: bool = False,
) -> Dict[str, List[str]]:
    ...

def list_files(
    group_by_ftype: bool = False,
    targets: Sequence[str] = [],
    ftypes: Sequence[str] = [],
    use_shebang: bool = True,
    modified_only: bool = False,
    exclude: Sequence[str] = [],
    extless_only: bool = False,
) -> Union[Dict[str, List[str]], List[str]]:
    """
    List files tracked by git.

    Returns a list of files which are either in targets or in directories in targets.
    If targets is [], list of all tracked files in current directory is returned.

    Other arguments:
    ftypes - List of file types on which to filter the search.
        If ftypes is [], all files are included.
    use_shebang - Determine file type of extensionless files from their shebang.
    modified_only - Only include files which have been modified.
    exclude - List of files or directories to be excluded, relative to repository root.
    group_by_ftype - If True, returns a dict of lists keyed by file type.
        If False, returns a flat list of files.
    extless_only - Only include extensionless files in output.
    """
    ftypes = [x.strip('.') for x in ftypes]
    ftypes_set = set(ftypes)

    # Really this is all bytes -- it's a file path -- but we get paths in
    # sys.argv as str, so that battle is already lost.  Settle for hoping
    # everything is UTF-8.
    repository_root = subprocess.check_output(['git', 'rev-parse',
                                               '--show-toplevel']).strip().decode('utf-8')
    exclude_abspaths = [os.path.abspath(os.path.join(repository_root, fpath)) for fpath in exclude]

    cmdline = ["git", "ls-files", "-z", *targets]
    if modified_only:
        cmdline.append('-m')

    files = [f.decode() for f in subprocess.check_output(cmdline).split(b"\0")]
    assert files.pop() == ""
    # throw away non-files (like symlinks)
    files = [f for f in files if os.path.isfile(f)]

    result_dict = defaultdict(list)  # type: Dict[str, List[str]]
    result_list = []  # type: List[str]

    for fpath in files:
        # this will take a long time if exclude is very large
        ext = os.path.splitext(fpath)[1]
        if extless_only and ext:
            continue
        absfpath = os.path.abspath(fpath)
        if any(absfpath == expath or absfpath.startswith(os.path.abspath(expath) + os.sep)
               for expath in exclude_abspaths):
            continue

        if ftypes or group_by_ftype:
            try:
                filetype = get_ftype(fpath, use_shebang)
            except (OSError, UnicodeDecodeError) as e:
                etype = e.__class__.__name__
                print('Error: %s while determining type of file "%s":' % (etype, fpath), file=sys.stderr)
                print(e, file=sys.stderr)
                filetype = ''
            if ftypes and filetype not in ftypes_set:
                continue

        if group_by_ftype:
            result_dict[filetype].append(fpath)
        else:
            result_list.append(fpath)

    if group_by_ftype:
        return result_dict
    else:
        return result_list

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="List files tracked by git and optionally filter by type")
    parser.add_argument('targets', nargs='*', default=[],
                        help='''files and directories to include in the result.
                        If this is not specified, the current directory is used''')
    parser.add_argument('-m', '--modified', action='store_true', default=False,
                        help='list only modified files')
    parser.add_argument('-f', '--ftypes', nargs='+', default=[],
                        help="list of file types to filter on. "
                             "All files are included if this option is absent")
    parser.add_argument('--ext-only', dest='extonly', action='store_true', default=False,
                        help='only use extension to determine file type')
    parser.add_argument('--exclude', nargs='+', default=[],
                        help='list of files and directories to exclude from results, relative to repo root')
    parser.add_argument('--extless-only', dest='extless_only', action='store_true', default=False,
                        help='only include extensionless files in output')
    args = parser.parse_args()
    listing = list_files(targets=args.targets, ftypes=args.ftypes, use_shebang=not args.extonly,
                         modified_only=args.modified, exclude=args.exclude, extless_only=args.extless_only)
    for l in listing:
        print(l)



from setuptools import find_packages, setup

VERSION="1.0.0"

REQUIRED = [
    'pyflakes',
    'pycodestyle',
    'typing-extensions',
]

def long_description() -> str:
    with open('README.md', encoding='utf8') as f:
        return f.read()

setup(
    name='adamlint',
    version=VERSION,
    author='ADB Web Designs',
    description='A linter launcher for projects that have multiple source languages.',
    long_description=long_description(),
    long_description_content_type='text/markdown',
    author_email='enquiries@adbwebdesigns.co.uk',
    python_requires='>=3.5',
    url='https://github.com/adambirds/adamlint',
    packages=find_packages(exclude=('tests',)),
    package_data={
        'zulint': ["py.typed"],
    },
    install_requires=REQUIRED,
    license='Apache License 2.0',
    classifiers=[
        'Development Status :: 4 - Beta',

        'Intended Audience :: Developers',
        'Topic :: Software Development :: Quality Assurance',
        'Topic :: Software Development :: Testing',

        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
)
